@@ -18,6 +18,8 @@
 
    Written by Paul Eggert.  */
 
+#include <sys/types.h>
+
 #if HAVE_SYS_ACL_H
 # include <sys/acl.h>
 #endif
@@ -25,6 +27,8 @@
 # define GETACLCNT ACL_CNT
 #endif
 
+#include <sys/stat.h>
+
 int file_has_acl (char const *, struct stat const *);
 int copy_acl (char const *, int, char const *, int, mode_t);
 int set_acl (char const *, int, mode_t);
