@@ -55,7 +55,7 @@
 char *program_name;
 
 /* Name of input file.  May be "-".  */
-static char *infile;
+static char const *infile;
 
 /* Descriptor on which input file is open.  */
 static int input_desc;
@@ -571,7 +571,9 @@ main (int argc, char **argv)
   textdomain (PACKAGE);
 
   /* Arrange to close stdout if we exit via the
-     case_GETOPT_HELP_CHAR or case_GETOPT_VERSION_CHAR code.  */
+     case_GETOPT_HELP_CHAR or case_GETOPT_VERSION_CHAR code.
+     Normally STDOUT_FILENO is used rather than stdout, so
+     close_stdout does nothing.  */
   atexit (close_stdout);
 
   /* Parse command line options.  */
