@@ -38,6 +38,7 @@
 #include "euidaccess.h"
 #include "error.h"
 #include "fcntl--.h"
+#include "filemode.h"
 #include "filenamecat.h"
 #include "full-write.h"
 #include "getpagesize.h"
@@ -797,10 +798,14 @@ overwrite_prompt (char const *dst_name, struct stat const *dst_sb)
 {
   if (euidaccess (dst_name, W_OK) != 0)
     {
+      char perms[12];		/* "-rwxrwxrwx " ls-style modes. */
+      strmode (dst_sb->st_mode, perms);
+      perms[10] = '\0';
       fprintf (stderr,
-	       _("%s: overwrite %s, overriding mode %04lo? "),
+	       _("%s: try to overwrite %s, overriding mode %04lo (%s)? "),
 	       program_name, quote (dst_name),
-	       (unsigned long int) (dst_sb->st_mode & CHMOD_MODE_BITS));
+	       (unsigned long int) (dst_sb->st_mode & CHMOD_MODE_BITS),
+	       &perms[1]);
     }
   else
     {
@@ -1604,19 +1609,20 @@ copy_internal (char const *src_name, char const *dst_name,
 	    emit_verbose (src_name, dst_name, NULL);
 	}
 
-      /* Are we crossing a file system boundary?  */
+      /* Decide whether to copy the contents of the directory.  */
       if (x->one_file_system && device != 0 && device != src_sb.st_dev)
-	return true;
-
-      /* Copy the contents of the directory.  */
-
-      if (! copy_dir (src_name, dst_name, new_dst, &src_sb, dir, x,
-		      copy_into_self))
 	{
-	  /* Don't just return here -- otherwise, the failure to read a
-	     single file in a source directory would cause the containing
-	     destination directory not to have owner/perms set properly.  */
-	  delayed_ok = false;
+	  /* Here, we are crossing a file system boundary and cp's -x option
+	     is in effect: so don't copy the contents of this directory. */
+	}
+      else
+	{
+	  /* Copy the contents of the directory.  Don't just return if
+	     this fails -- otherwise, the failure to read a single file
+	     in a source directory would cause the containing destination
+	     directory not to have owner/perms set properly.  */
+	  delayed_ok = copy_dir (src_name, dst_name, new_dst, &src_sb, dir, x,
+				 copy_into_self);
 	}
     }
   else if (x->symbolic_link)
@@ -1720,7 +1726,7 @@ copy_internal (char const *src_name, char const *dst_name,
     }
   else if (S_ISLNK (src_mode))
     {
-      char *src_link_val = xreadlink (src_name, src_sb.st_size);
+      char *src_link_val = xreadlink_with_size (src_name, src_sb.st_size);
       if (src_link_val == NULL)
 	{
 	  error (0, errno, _("cannot read symbolic link %s"), quote (src_name));
@@ -1740,7 +1746,8 @@ copy_internal (char const *src_name, char const *dst_name,
 		 FIXME: This behavior isn't documented, and seems wrong
 		 in some cases, e.g., if the destination symlink has the
 		 wrong ownership, permissions, or time stamps.  */
-	      char *dest_link_val = xreadlink (dst_name, dst_sb.st_size);
+	      char *dest_link_val =
+		xreadlink_with_size (dst_name, dst_sb.st_size);
 	      if (STREQ (dest_link_val, src_link_val))
 		same_link = true;
 	      free (dest_link_val);
