@@ -1,5 +1,5 @@
 /* cp.c  -- file copying (main routines)
-   Copyright (C) 89, 90, 91, 1995-2006 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 1995-2007 Free Software Foundation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -413,6 +413,9 @@ make_dir_parents_private (char const *const_dir, size_t src_offset,
 	  if (XSTAT (x, dir, &stats))
 	    {
 	      mode_t src_mode;
+	      mode_t omitted_permissions;
+	      mode_t mkdir_mode;
+	      int src_errno;
 
 	      /* This component does not exist.  We must set
 		 *new_dst and new->mode inside this loop because,
@@ -420,15 +423,35 @@ make_dir_parents_private (char const *const_dir, size_t src_offset,
 		 make_dir_parents_private creates only e_dir/../a if
 		 ./b already exists. */
 	      *new_dst = true;
-	      if (XSTAT (x, src, &stats))
+	      src_errno = (XSTAT (x, src, &stats) != 0
+			   ? errno
+			   : S_ISDIR (stats.st_mode)
+			   ? 0
+			   : ENOTDIR);
+	      if (src_errno)
 		{
-		  error (0, errno, _("failed to get attributes of %s"),
+		  error (0, src_errno, _("failed to get attributes of %s"),
 			 quote (src));
 		  return false;
 		}
 	      src_mode = stats.st_mode;
 
-	      if (mkdir (dir, src_mode))
+	      /* If the ownership or special mode bits might change,
+		 omit some permissions at first, so unauthorized users
+		 cannot nip in before the file is ready.  */
+	      omitted_permissions = (src_mode
+				     & (x->preserve_ownership
+					? S_IRWXG | S_IRWXO
+					: x->preserve_mode
+					? S_IWGRP | S_IWOTH
+					: 0));
+
+	      /* POSIX says mkdir's behavior is implementation-defined when
+		 (src_mode & ~S_IRWXUGO) != 0.  However, common practice is
+		 to ask mkdir to copy all the CHMOD_MODE_BITS, letting mkdir
+		 decide what to do with S_ISUID | S_ISGID | S_ISVTX.  */
+	      mkdir_mode = src_mode & CHMOD_MODE_BITS & ~omitted_permissions;
+	      if (mkdir (dir, mkdir_mode) != 0)
 		{
 		  error (0, errno, _("cannot make directory %s"),
 			 quote (dir));
@@ -450,28 +473,30 @@ make_dir_parents_private (char const *const_dir, size_t src_offset,
 			 quote (dir));
 		  return false;
 		}
-	      else
-	        {
-		  if (x->preserve_mode)
-		    {
-		      new->mode = src_mode;
-		      new->restore_mode = (src_mode != stats.st_mode);
-		    }
 
-		  if ((stats.st_mode & S_IRWXU) != S_IRWXU)
-		    {
-		      /* Make the new directory searchable and writable. The
-			 original permissions will be restored later.  */
 
-		      new->mode = stats.st_mode;
+	      if (! x->preserve_mode)
+		{
+		  if (omitted_permissions & ~stats.st_mode)
+		    omitted_permissions &= ~ cached_umask ();
+		  if (omitted_permissions & ~stats.st_mode
+		      || (stats.st_mode & S_IRWXU) != S_IRWXU)
+		    {
+		      new->mode = stats.st_mode | omitted_permissions;
 		      new->restore_mode = true;
+		    }
+		}
 
-		      if (lchmod (dir, stats.st_mode | S_IRWXU) != 0)
-			{
-			  error (0, errno, _("setting permissions for %s"),
-				 quote (dir));
-			  return false;
-			}
+	      if ((stats.st_mode & S_IRWXU) != S_IRWXU)
+		{
+		  /* Make the new directory searchable and writable.
+		     The original permissions will be restored later.  */
+
+		  if (lchmod (dir, stats.st_mode | S_IRWXU) != 0)
+		    {
+		      error (0, errno, _("setting permissions for %s"),
+			     quote (dir));
+		      return false;
 		    }
 		}
 	    }
