@@ -1,5 +1,5 @@
 /* dd -- convert a file while copying it.
-   Copyright (C) 85, 90, 91, 1995-2006 Free Software Foundation, Inc.
+   Copyright (C) 85, 90, 91, 1995-2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -267,7 +267,7 @@ static struct symbol_value const flags[] =
   {"dsync",	O_DSYNC},
   {"noatime",	O_NOATIME},
   {"noctty",	O_NOCTTY},
-  {"nofollow",	O_NOFOLLOW},
+  {"nofollow",	HAVE_WORKING_O_NOFOLLOW ? O_NOFOLLOW : 0},
   {"nolinks",	O_NOLINKS},
   {"nonblock",	O_NONBLOCK},
   {"sync",	O_SYNC},
@@ -448,10 +448,12 @@ Each CONV symbol may be:\n\
   notrunc   do not truncate the output file\n\
   ucase     change lower case to upper case\n\
   swab      swap every pair of input bytes\n\
+"), stdout);
+      fputs (_("\
   noerror   continue after read errors\n\
   sync      pad every input block with NULs to ibs-size; when used\n\
-              with block or unblock, pad with spaces rather than NULs\n\
-  fdatasync physically write output file data before finishing\n\
+            with block or unblock, pad with spaces rather than NULs\n\
+  fdatasync  physically write output file data before finishing\n\
   fsync     likewise, but also write metadata\n\
 "), stdout);
       fputs (_("\
@@ -475,7 +477,7 @@ Each FLAG symbol may be:\n\
       if (O_NOCTTY)
 	fputs (_("  noctty    do not assign controlling terminal from file\n"),
 	       stdout);
-      if (O_NOFOLLOW)
+      if (HAVE_WORKING_O_NOFOLLOW)
 	fputs (_("  nofollow  do not follow symlinks\n"), stdout);
       if (O_NOLINKS)
 	fputs (_("  nolinks   fail if multiply-linked\n"), stdout);
@@ -871,6 +873,7 @@ static void
 scanargs (int argc, char **argv)
 {
   int i;
+  size_t blocksize = 0;
 
   for (i = optind; i < argc; i++)
     {
@@ -921,7 +924,7 @@ scanargs (int argc, char **argv)
 	  else if (STREQ (name, "bs"))
 	    {
 	      invalid |= ! (0 < n && n <= MAX_BLOCKSIZE (INPUT_BLOCK_SLOP));
-	      output_blocksize = input_blocksize = n;
+	      blocksize = n;
 	    }
 	  else if (STREQ (name, "cbs"))
 	    {
@@ -946,6 +949,9 @@ scanargs (int argc, char **argv)
 	}
     }
 
+  if (blocksize)
+    input_blocksize = output_blocksize = blocksize;
+
   /* If bs= was given, both `input_blocksize' and `output_blocksize' will
      have been set to positive values.  If either has not been set,
      bs= was not given, so make sure two buffers are used. */
@@ -1220,7 +1226,7 @@ advance_input_after_read_error (size_t nbytes)
 	    return true;
 	  diff = input_offset - offset;
 	  if (! (0 <= diff && diff <= nbytes))
-	    error (0, 0, _("warning: screwy file offset after failed read"));
+	    error (0, 0, _("warning: invalid file offset after failed read"));
 	  if (0 <= skip_via_lseek (input_file, STDIN_FILENO, diff, SEEK_CUR))
 	    return true;
 	  if (errno == 0)
@@ -1377,8 +1383,10 @@ static int
 dd_copy (void)
 {
   char *ibuf, *bufstart;	/* Input buffer. */
-  char *real_buf;		/* real buffer address before alignment */
-  char *real_obuf;
+  /* These are declared static so that even though we don't free the
+     buffers, valgrind will recognize that there is no "real" leak.  */
+  static char *real_buf;	/* real buffer address before alignment */
+  static char *real_obuf;
   ssize_t nread;		/* Bytes read in the current block.  */
 
   /* If nonzero, then the previously read block was partial and
@@ -1598,9 +1606,6 @@ dd_copy (void)
 	}
     }
 
-  free (real_buf);
-  free (real_obuf);
-
   if ((conversions_mask & C_FDATASYNC) && fdatasync (STDOUT_FILENO) != 0)
     {
       if (errno != ENOSYS && errno != EINVAL)
