@@ -1,5 +1,5 @@
 /* expr -- evaluate expressions.
-   Copyright (C) 86, 1991-1997, 1999-2004 Free Software Foundation, Inc.
+   Copyright (C) 86, 1991-1997, 1999-2006 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Author: Mike Parker.
 
@@ -37,26 +37,26 @@
 #include "long-options.h"
 #include "error.h"
 #include "inttostr.h"
+#include "quote.h"
 #include "quotearg.h"
+#include "strnumcmp.h"
+#include "xstrtol.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "expr"
 
 #define AUTHORS "Mike Parker"
 
-#undef NEW
-#define NEW(Type) XMALLOC (Type, 1)
-#define OLD(x) free (x)
-
 /* Exit statuses.  */
 enum
   {
-    /* Invalid expression: i.e., its form does not conform to the
+    /* Invalid expression: e.g., its form does not conform to the
        grammar for expressions.  Our grammar is an extension of the
        POSIX grammar.  */
     EXPR_INVALID = 2,
 
-    /* Some other error occurred.  */
+    /* An internal error occurred, e.g., arithmetic overflow, storage
+       exhaustion.  */
     EXPR_FAILURE
   };
 
@@ -157,6 +157,11 @@ Beware that many operators need to be escaped or quoted for shells.\n\
 Comparisons are arithmetic if both ARGs are numbers, else lexicographical.\n\
 Pattern matches return the string matched between \\( and \\) or null; if\n\
 \\( and \\) are not used, they return the number of characters matched or 0.\n\
+"), stdout);
+      fputs (_("\
+\n\
+Exit status is 0 if EXPRESSION is neither null nor 0, 1 if EXPRESSION is null\n\
+or 0, 2 if EXPRESSION is syntactically invalid, and 3 if an error occurred.\n\
 "), stdout);
       printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
@@ -170,6 +175,13 @@ syntax_error (void)
   error (EXPR_INVALID, 0, _("syntax error"));
 }
 
+/* Report an integer overflow for operation OP and exit.  */
+static void
+integer_overflow (char op)
+{
+  error (EXPR_FAILURE, ERANGE, "%c", op);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -196,7 +208,7 @@ main (int argc, char **argv)
 
   if (argc <= 1)
     {
-      error (0, 0, _("too few arguments"));
+      error (0, 0, _("missing operand"));
       usage (EXPR_INVALID);
     }
 
@@ -215,9 +227,7 @@ main (int argc, char **argv)
 static VALUE *
 int_value (intmax_t i)
 {
-  VALUE *v;
-
-  v = NEW (VALUE);
+  VALUE *v = xmalloc (sizeof *v);
   v->type = integer;
   v->u.i = i;
   return v;
@@ -226,11 +236,9 @@ int_value (intmax_t i)
 /* Return a VALUE for S.  */
 
 static VALUE *
-str_value (char *s)
+str_value (char const *s)
 {
-  VALUE *v;
-
-  v = NEW (VALUE);
+  VALUE *v = xmalloc (sizeof *v);
   v->type = string;
   v->u.s = xstrdup (s);
   return v;
@@ -243,7 +251,7 @@ freev (VALUE *v)
 {
   if (v->type == string)
     free (v->u.s);
-  OLD (v);
+  free (v);
 }
 
 /* Print VALUE V.  */
@@ -252,7 +260,7 @@ static void
 printv (VALUE *v)
 {
   char *p;
-  char buf[INT_STRLEN_BOUND (intmax_t) + 1];
+  char buf[INT_BUFSIZE_BOUND (intmax_t)];
 
   switch (v->type)
     {
@@ -300,12 +308,27 @@ null (VALUE *v)
     }
 }
 
+/* Return true if CP takes the form of an integer.  */
+
+static bool
+looks_like_integer (char const *cp)
+{
+  cp += (*cp == '-');
+
+  do
+    if (! ISDIGIT (*cp))
+      return false;
+  while (*++cp);
+
+  return true;
+}
+
 /* Coerce V to a string value (can't fail).  */
 
 static void
 tostring (VALUE *v)
 {
-  char buf[INT_STRLEN_BOUND (intmax_t) + 1];
+  char buf[INT_BUFSIZE_BOUND (intmax_t)];
 
   switch (v->type)
     {
@@ -325,50 +348,39 @@ tostring (VALUE *v)
 static bool
 toarith (VALUE *v)
 {
-  intmax_t i;
-  bool neg;
-  char *cp;
-
   switch (v->type)
     {
     case integer:
       return true;
     case string:
-      i = 0;
-      cp = v->u.s;
-      neg = (*cp == '-');
-      if (neg)
-	cp++;
+      {
+	intmax_t value;
 
-      do
-	{
-	  if (ISDIGIT (*cp))
-	    i = i * 10 + *cp - '0';
-	  else
-	    return false;
-	}
-      while (*++cp);
-
-      free (v->u.s);
-      v->u.i = i * (neg ? -1 : 1);
-      v->type = integer;
-      return true;
+	if (! looks_like_integer (v->u.s))
+	  return false;
+	if (xstrtoimax (v->u.s, NULL, 10, &value, NULL) != LONGINT_OK)
+	  error (EXPR_FAILURE, ERANGE, "%s", v->u.s);
+	free (v->u.s);
+	v->u.i = value;
+	v->type = integer;
+	return true;
+      }
     default:
       abort ();
     }
 }
 
-/* Return nonzero and advance if the next token matches STR exactly.
+/* Return true and advance if the next token matches STR exactly.
    STR must not be NULL.  */
 
 static bool
 nextarg (char const *str)
 {
   if (*args == NULL)
-    return 0;
+    return false;
   else
     {
-      bool r = strcmp (*args, str) == 0;
+      bool r = STREQ (*args, str);
       args += r;
       return r;
     }
@@ -405,42 +417,36 @@ trace (fxn)
 static VALUE *
 docolon (VALUE *sv, VALUE *pv)
 {
-  VALUE *v;
+  VALUE *v IF_LINT (= NULL);
   const char *errmsg;
   struct re_pattern_buffer re_buffer;
+  char fastmap[UCHAR_MAX + 1];
   struct re_registers re_regs;
-  size_t len;
-  int matchlen;
+  regoff_t matchlen;
 
   tostring (sv);
   tostring (pv);
 
-  if (pv->u.s[0] == '^')
-    {
-      error (0, 0, _("\
-warning: unportable BRE: `%s': using `^' as the first character\n\
-of the basic regular expression is not portable; it is being ignored"),
-	     pv->u.s);
-    }
+  re_regs.num_regs = 0;
+  re_regs.start = NULL;
+  re_regs.end = NULL;
 
-  len = strlen (pv->u.s);
-  memset (&re_buffer, 0, sizeof (re_buffer));
-  memset (&re_regs, 0, sizeof (re_regs));
-  re_buffer.allocated = 2 * len;
-  if (re_buffer.allocated < len)
-    xalloc_die ();
-  re_buffer.buffer = xmalloc (re_buffer.allocated);
-  re_buffer.translate = 0;
-  re_syntax_options = RE_SYNTAX_POSIX_BASIC;
-  errmsg = re_compile_pattern (pv->u.s, len, &re_buffer);
+  re_buffer.buffer = NULL;
+  re_buffer.allocated = 0;
+  re_buffer.fastmap = fastmap;
+  re_buffer.translate = NULL;
+  re_syntax_options =
+    RE_SYNTAX_POSIX_BASIC & ~RE_CONTEXT_INVALID_DUP & ~RE_NO_EMPTY_RANGES;
+  errmsg = re_compile_pattern (pv->u.s, strlen (pv->u.s), &re_buffer);
   if (errmsg)
-    error (EXPR_FAILURE, 0, "%s", errmsg);
+    error (EXPR_INVALID, 0, "%s", errmsg);
+  re_buffer.newline_anchor = 0;
 
   matchlen = re_match (&re_buffer, sv->u.s, strlen (sv->u.s), 0, &re_regs);
   if (0 <= matchlen)
     {
       /* Were \(...\) used? */
-      if (re_buffer.re_nsub > 0)/* was (re_regs.start[1] >= 0) */
+      if (re_buffer.re_nsub > 0)
 	{
 	  sv->u.s[re_regs.end[1]] = '\0';
 	  v = str_value (sv->u.s + re_regs.start[1]);
@@ -448,7 +454,7 @@ of the basic regular expression is not portable; it is being ignored"),
       else
 	v = int_value (matchlen);
     }
-  else
+  else if (matchlen == -1)
     {
       /* Match failed -- return the right kind of null.  */
       if (re_buffer.re_nsub > 0)
@@ -456,7 +462,18 @@ of the basic regular expression is not portable; it is being ignored"),
       else
 	v = int_value (0);
     }
-  free (re_buffer.buffer);
+  else
+    error (EXPR_FAILURE,
+	   (matchlen == -2 ? errno : EOVERFLOW),
+	   _("error in regular expression matcher"));
+
+  if (0 < re_regs.num_regs)
+    {
+      free (re_regs.start);
+      free (re_regs.end);
+    }
+  re_buffer.fastmap = NULL;
+  regfree (&re_buffer);
   return v;
 }
 
@@ -544,21 +561,25 @@ eval6 (bool evaluate)
     }
   else if (nextarg ("substr"))
     {
+      size_t llen;
       l = eval6 (evaluate);
       i1 = eval6 (evaluate);
       i2 = eval6 (evaluate);
       tostring (l);
+      llen = strlen (l->u.s);
       if (!toarith (i1) || !toarith (i2)
-	  || strlen (l->u.s) < i1->u.i
+	  || llen < i1->u.i
 	  || i1->u.i <= 0 || i2->u.i <= 0)
 	v = str_value ("");
       else
 	{
-	  v = NEW (VALUE);
+	  size_t vlen = MIN (i2->u.i, llen - i1->u.i + 1);
+	  char *vlim;
+	  v = xmalloc (sizeof *v);
 	  v->type = string;
-	  v->u.s = strncpy (xmalloc (i2->u.i + 1),
-			    l->u.s + i1->u.i - 1, i2->u.i);
-	  v->u.s[i2->u.i] = 0;
+	  v->u.s = xmalloc (vlen + 1);
+	  vlim = mempcpy (v->u.s, l->u.s + i1->u.i - 1, vlen);
+	  *vlim = '\0';
 	}
       freev (l);
       freev (i1);
@@ -629,14 +650,30 @@ eval4 (bool evaluate)
       if (evaluate)
 	{
 	  if (!toarith (l) || !toarith (r))
-	    error (EXPR_FAILURE, 0, _("non-numeric argument"));
+	    error (EXPR_INVALID, 0, _("non-numeric argument"));
 	  if (fxn == multiply)
-	    val = l->u.i * r->u.i;
+	    {
+	      val = l->u.i * r->u.i;
+	      if (! (l->u.i == 0 || r->u.i == 0
+		     || ((val < 0) == ((l->u.i < 0) ^ (r->u.i < 0))
+			 && val / l->u.i == r->u.i)))
+		integer_overflow ('*');
+	    }
 	  else
 	    {
 	      if (r->u.i == 0)
-		error (EXPR_FAILURE, 0, _("division by zero"));
-	      val = fxn == divide ? l->u.i / r->u.i : l->u.i % r->u.i;
+		error (EXPR_INVALID, 0, _("division by zero"));
+	      if (l->u.i < - INTMAX_MAX && r->u.i == -1)
+		{
+		  /* Some x86-style hosts raise an exception for
+		     INT_MIN / -1 and INT_MIN % -1, so handle these
+		     problematic cases specially.  */
+		  if (fxn == divide)
+		    integer_overflow ('/');
+		  val = 0;
+		}
+	      else
+		val = fxn == divide ? l->u.i / r->u.i : l->u.i % r->u.i;
 	    }
 	}
       freev (l);
@@ -671,8 +708,19 @@ eval3 (bool evaluate)
       if (evaluate)
 	{
 	  if (!toarith (l) || !toarith (r))
-	    error (EXPR_FAILURE, 0, _("non-numeric argument"));
-	  val = fxn == plus ? l->u.i + r->u.i : l->u.i - r->u.i;
+	    error (EXPR_INVALID, 0, _("non-numeric argument"));
+	  if (fxn == plus)
+	    {
+	      val = l->u.i + r->u.i;
+	      if ((val < l->u.i) != (r->u.i < 0))
+		integer_overflow ('+');
+	    }
+	  else
+	    {
+	      val = l->u.i - r->u.i;
+	      if ((l->u.i < val) != (r->u.i < 0))
+		integer_overflow ('-');
+	    }
 	}
       freev (l);
       freev (r);
@@ -686,16 +734,6 @@ static VALUE *
 eval2 (bool evaluate)
 {
   VALUE *l;
-  VALUE *r;
-  enum
-  {
-    less_than, less_equal, equal, not_equal, greater_equal, greater_than
-  } fxn;
-  bool val;
-  intmax_t lval;
-  intmax_t rval;
-  int collation_errno;
-  char *collation_arg1;
 
 #ifdef EVAL_TRACE
   trace ("eval2");
@@ -703,6 +741,13 @@ eval2 (bool evaluate)
   l = eval3 (evaluate);
   while (1)
     {
+      VALUE *r;
+      enum
+	{
+	  less_than, less_equal, equal, not_equal, greater_equal, greater_than
+	} fxn;
+      bool val = false;
+
       if (nextarg ("<"))
 	fxn = less_than;
       else if (nextarg ("<="))
@@ -718,46 +763,45 @@ eval2 (bool evaluate)
       else
 	return l;
       r = eval3 (evaluate);
-      tostring (l);
-      tostring (r);
 
-      /* Save the first arg to strcoll, in case we need its value for
-	 a diagnostic later.  This is needed because 'toarith' might
-	 free the first arg.  */
-      collation_arg1 = xstrdup (l->u.s);
+      if (evaluate)
+	{
+	  int cmp;
+	  tostring (l);
+	  tostring (r);
 
-      errno = 0;
-      lval = strcoll (collation_arg1, r->u.s);
-      collation_errno = errno;
-      rval = 0;
-      if (toarith (l) && toarith (r))
-	{
-	  lval = l->u.i;
-	  rval = r->u.i;
-	}
-      else if (collation_errno && evaluate)
-	{
-	  error (0, collation_errno, _("string comparison failed"));
-	  error (0, 0, _("Set LC_ALL='C' to work around the problem."));
-	  error (EXPR_FAILURE, 0,
-		 _("The strings compared were %s and %s."),
-		 quotearg_n_style (0, locale_quoting_style, collation_arg1),
-		 quotearg_n_style (1, locale_quoting_style, r->u.s));
+	  if (looks_like_integer (l->u.s) && looks_like_integer (r->u.s))
+	    cmp = strintcmp (l->u.s, r->u.s);
+	  else
+	    {
+	      errno = 0;
+	      cmp = strcoll (l->u.s, r->u.s);
+
+	      if (errno)
+		{
+		  error (0, errno, _("string comparison failed"));
+		  error (0, 0, _("Set LC_ALL='C' to work around the problem."));
+		  error (EXPR_INVALID, 0,
+			 _("The strings compared were %s and %s."),
+			 quotearg_n_style (0, locale_quoting_style, l->u.s),
+			 quotearg_n_style (1, locale_quoting_style, r->u.s));
+		}
+	    }
+
+	  switch (fxn)
+	    {
+	    case less_than:     val = (cmp <  0); break;
+	    case less_equal:    val = (cmp <= 0); break;
+	    case equal:         val = (cmp == 0); break;
+	    case not_equal:     val = (cmp != 0); break;
+	    case greater_equal: val = (cmp >= 0); break;
+	    case greater_than:  val = (cmp >  0); break;
+	    default: abort ();
+	    }
 	}
 
-      switch (fxn)
-	{
-	case less_than:     val = (lval <  rval); break;
-	case less_equal:    val = (lval <= rval); break;
-	case equal:         val = (lval == rval); break;
-	case not_equal:     val = (lval != rval); break;
-	case greater_equal: val = (lval >= rval); break;
-	case greater_than:  val = (lval >  rval); break;
-	default: abort ();
-	}
       freev (l);
       freev (r);
-      free (collation_arg1);
       l = int_value (val);
     }
 }
