@@ -928,7 +928,7 @@ open_next_file (void)
   while (in_stream == NULL);
 
   if (limit_bytes_to_format & !flag_dump_strings)
-    SETVBUF (in_stream, NULL, _IONBF, 0);
+    setvbuf (in_stream, NULL, _IONBF, 0);
 
   return ok;
 }
