@@ -1,5 +1,5 @@
 /* readlink -- display value of a symbolic link.
-   Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -74,6 +74,8 @@ usage (int status)
   -e, --canonicalize-existing   canonicalize by following every symlink in\n\
                                 every component of the given name recursively,\n\
                                 all components must exist\n\
+"), stdout);
+      fputs (_("\
   -m, --canonicalize-missing    canonicalize by following every symlink in\n\
                                 every component of the given name recursively,\n\
                                 without requirements on components existence\n\
@@ -157,7 +159,7 @@ main (int argc, char **argv)
 
   value = (can_mode != -1
 	   ? canonicalize_filename_mode (fname, can_mode)
-	   : xreadlink (fname, 1024));
+	   : xreadlink (fname));
   if (value)
     {
       printf ("%s%s", value, (no_newline ? "" : "\n"));
